# -*- coding: utf-8 -*-
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import google.api_core.grpc_helpers

from google.android.overtheair_v1.proto import overtheair_service_pb2_grpc


class OverTheAirServiceGrpcTransport(object):
    """gRPC transport class providing stubs for
    google.android.overtheair.v1 OverTheAirService API.

    The transport provides access to the raw gRPC stubs,
    which can be used to take advantage of advanced
    features of gRPC.
    """
    # The scopes needed to make gRPC calls to all of the methods defined
    # in this service.
    _OAUTH_SCOPES = (
        'https://www.googleapis.com/auth/android_partner_over_the_air',
        'https://www.googleapis.com/auth/userinfo.email',
    )

    def __init__(self,
                 channel=None,
                 credentials=None,
                 address='androidovertheair.googleapis.com:443'):
        """Instantiate the transport class.

        Args:
            channel (grpc.Channel): A ``Channel`` instance through
                which to make calls. This argument is mutually exclusive
                with ``credentials``; providing both will raise an exception.
            credentials (google.auth.credentials.Credentials): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            address (str): The address where the service is hosted.
        """
        # If both `channel` and `credentials` are specified, raise an
        # exception (channels come with credentials baked in already).
        if channel is not None and credentials is not None:
            raise ValueError(
                'The `channel` and `credentials` arguments are mutually '
                'exclusive.', )

        # Create the channel.
        if channel is None:
            channel = self.create_channel(
                address=address,
                credentials=credentials,
            )

        self._channel = channel

        # gRPC uses objects called "stubs" that are bound to the
        # channel and provide a basic method for each RPC.
        self._stubs = {
            'over_the_air_service_stub':
            overtheair_service_pb2_grpc.OverTheAirServiceStub(channel),
        }

    @classmethod
    def create_channel(cls,
                       address='androidovertheair.googleapis.com:443',
                       credentials=None):
        """Create and return a gRPC channel object.

        Args:
            address (str): The host for the channel to use.
            credentials (~.Credentials): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.

        Returns:
            grpc.Channel: A gRPC channel object.
        """
        return google.api_core.grpc_helpers.create_channel(
            address,
            credentials=credentials,
            scopes=cls._OAUTH_SCOPES,
        )

    @property
    def channel(self):
        """The gRPC channel used by the transport.

        Returns:
            grpc.Channel: A gRPC channel object.
        """
        return self._channel

    @property
    def get_deployment(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.get_deployment`.

        Gets a ``Deployment``. Returns ``NOT_FOUND`` if the deployment does not
        exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].GetDeployment

    @property
    def list_deployments(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.list_deployments`.

        Lists ``deployments``.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].ListDeployments

    @property
    def update_deployment(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.update_deployment`.

        Updates a ``deployment``.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].UpdateDeployment

    @property
    def get_package(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.get_package`.

        Gets a ``package``. Returns ``NOT_FOUND`` if the matching path does not
        exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].GetPackage

    @property
    def list_packages(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.list_packages`.

        Lists ``packages``.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].ListPackages

    @property
    def delete_package(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.delete_package`.

        Deletes a ``package``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].DeletePackage

    @property
    def update_package(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.update_package`.

        Updates a ``package``. Returns ``INVALID_ARGUMENT`` if any field other
        than title is modified.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].UpdatePackage

    @property
    def create_config(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.create_config`.

        Creates a config, and returns the new ``Config``.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].CreateConfig

    @property
    def get_config(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.get_config`.

        Gets a ``config`` view. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].GetConfig

    @property
    def list_configs(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.list_configs`.

        Lists ``configs`` of a deployment. Returns ``NOT_FOUND`` if the matching
        path does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].ListConfigs

    @property
    def delete_config(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.delete_config`.

        Deletes a ``config``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].DeleteConfig

    @property
    def update_config(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.update_config`.

        Updates a ``config``. Returns ``INVALID_ARGUMENT`` if the name of the
        config is non-empty and does not equal the existing name.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].UpdateConfig

    @property
    def create_group(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.create_group`.

        Creates a group, and returns the new ``Group``.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].CreateGroup

    @property
    def get_group(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.get_group`.

        Gets a ``group``. Returns ``NOT_FOUND`` if the group does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].GetGroup

    @property
    def list_groups(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.list_groups`.

        Lists ``groups`` of a deployment. Returns ``NOT_FOUND`` if the matching
        path does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].ListGroups

    @property
    def delete_group(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.delete_group`.

        Deletes a ``group``. Returns ``NOT_FOUND`` if the group does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].DeleteGroup

    @property
    def update_group(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.update_group`.

        Updates a ``group``. Returns INVALID\_ARGUMENT if the name of the group
        is non-empty and does not equal the existing name.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].UpdateGroup

    @property
    def disable_groups(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.disable_groups`.

        Disables the defined ``groups``. Returns ``NOT_FOUND`` if the matching
        path does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].DisableGroups

    @property
    def enable_groups(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.enable_groups`.

        Enables the defined ``groups``. Returns ``NOT_FOUND`` if the matching
        path does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].EnableGroups

    @property
    def create_group_member(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.create_group_member`.

        Creates a group member and returns the new ``GroupMember``. If the
        device is already under other groups, it will be removed from those
        groups and added to the group that's specified in the request.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].CreateGroupMember

    @property
    def create_group_members(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.create_group_members`.

        Creates group members in a specific group.  If any device is in other
        groups, it will be removed from those groups and added to the group that's
        specified in the request.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].CreateGroupMembers

    @property
    def get_group_member(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.get_group_member`.

        Gets a ``group member``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].GetGroupMember

    @property
    def list_group_members(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.list_group_members`.

        Lists ``group members``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].ListGroupMembers

    @property
    def delete_group_member(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.delete_group_member`.

        Deletes a ``group member``. Returns ``NOT_FOUND`` if the matching path
        does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].DeleteGroupMember

    @property
    def delete_group_members(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.delete_group_members`.

        Deletes group members in a group.  If any member does not it exist, it
        will be ignored.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].DeleteGroupMembers

    @property
    def get_group_chart(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.get_group_chart`.

        Gets a ``group chart``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].GetGroupChart

    @property
    def list_group_charts(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.list_group_charts`.

        Lists ``group charts``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].ListGroupCharts

    @property
    def list_group_configs(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.list_group_configs`.

        Lists configs of the group. Returns ``NOT_FOUND`` if the matching path
        does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].ListGroupConfigs

    @property
    def update_group_configs(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.update_group_configs`.

        Update the configs under a group. Returns ``NOT_FOUND`` if the matching
        group or config does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].UpdateGroupConfigs

    @property
    def disable_all_enabled_groups_of_config(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.disable_all_enabled_groups_of_config`.

        Disables all enabled groups that have the specified config. Returns
        ``NOT_FOUND`` if the matching path does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs[
            'over_the_air_service_stub'].DisableAllEnabledGroupsOfConfig

    @property
    def list_config_packages(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.list_config_packages`.

        Lists packages of the config. Returns ``NOT_FOUND`` if the matching path
        does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].ListConfigPackages

    @property
    def update_config_packages(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.update_config_packages`.

        Update the packages under the config. Returns ``NOT_FOUND`` if the
        matching path does not exist.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].UpdateConfigPackages

    @property
    def list_configs_using_package(self):
        """Return the gRPC stub for :meth:`OverTheAirServiceClient.list_configs_using_package`.

        Lists all the configs that use the specified package.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs['over_the_air_service_stub'].ListConfigsUsingPackage
